#! /usr/bin/ env python3

# 180413 update

########################################################################
###     IIIII   M   M   PPPP   OOO  RRRR  TTTTT  AAA  N   N TTTTT    ###
###       I    M M M M  P   P O   O R   R   T    A A  NN  N   T      ###
###       I    M  M  M  PPPP  O   O RRRR    T   AAAAA N N N   T      ###
###       I   M       M P     O   O R  R    T   A   A N  NN   T      ###
###     IIIII M       M P      OOO  R   R   T   A   A N   N   T      ###
########################################################################
### To run:                                                          ###
###     In Vim, type :%s/#imp//g, to remove commented out sections   ###  
########################################################################
### Command-Line Flags:                                              ###
###     -d    : debug (closes GUI instead of shutting down device)   ###
########################################################################
### TODO:                                                            ###
###     zf - Recieve values from uC after stop is pushed             ###
###     zf - Implement params.py for inputs                          ###
###     zf - re-add Easter eggs if time permits                      ###
########################################################################

import os
import sys
from enum import Enum
from PyQt5 import uic
from PyQt5 import QtCore
from PyQt5 import QtGui
from PyQt5.QtWidgets import *

from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *


# Use for each developed script to add its directory to path

sys.path.append( os.path.dirname( os.path.dirname( 
    os.path.abspath( "implement.py" ) ) ) )
sys.path.append( os.path.dirname( os.path.dirname(
    os.path.abspath( "params.py" ) ) ) )
sys.path.append( os.path.dirname( os.path.dirname(
    os.path.abspath( "i2c.py" ) ) ) )


import implement as imp
from params import *  

kOhm = " K\u03A9"

########################################################################
### Parameter selection lists: hard code these values                ###
########################################################################
easter_egg_code = []

uiDir = "ui/"

class MainWindow( QMainWindow ):
    def __init__( self ):
        super().__init__()

        self.width = 800
        self.height = 480

        self.vce_swp = "-1"
        self.vbe_swp = "-1"
        self.vrc_swp = "-1"
        self.vrb_swp = "-1"
    
        self.vds_swp = "-1"
        self.vgs_swp = "-1"
        self.vrd_swp = "-1"

        self.vd_swp = "-1"
        self.vr_swp = "-1"

        self.init()


    def init( self ):
        main_window = uic.loadUi( uiDir + "sdguiRev3.ui", self )
        
        # Figure out a better way to implement
        self.main_window = main_window

        # Initialize gui xml
        main_window.setFixedSize( self.width, self.height )
       
        # Initialize main page images
        main_window.label_main_bjt.setPixmap( QtGui.QPixmap( Image.home[ "BJT" ] ) )
        main_window.label_main_mosfet.setPixmap( QtGui.QPixmap( Image.home[ "MOSFET" ] ) )
        main_window.label_main_diode.setPixmap( QtGui.QPixmap( Image.home[ "Diode" ] ) )

        # Initialize component page images
        main_window.label_bjt_off_device.setPixmap( QtGui.QPixmap( Image.BJT[ "off" ] ) )
        main_window.label_bjt_on_device.setPixmap( QtGui.QPixmap( Image.BJT[ "on" ] ) )

        main_window.label_mosfet_off_device.setPixmap( QtGui.QPixmap( Image.MOSFET[ "off" ] ) )
        main_window.label_mosfet_on_device.setPixmap( QtGui.QPixmap( Image.MOSFET[ "on" ] ) )

        main_window.label_diode_off_device.setPixmap( QtGui.QPixmap( Image.Diode[ "off" ] ) )
        main_window.label_diode_on_device.setPixmap( QtGui.QPixmap( Image.Diode[ "on" ] ) )

        # Initialize I/O
        self.add_functionality( main_window )

        # Set initial values for widgets
        main_window.stacked_widget.setCurrentIndex( Page.MAIN.value )
        self.showFullScreen()

    def banner( self, string ):
        label = self.main_window.label_banner

        label.setText( string ) 
        label.setAlignment( Qt.AlignHCenter | Qt.AlignBottom )
        label.setStyleSheet( "QLabel { color : white }" )
        label.setFont( QtGui.QFont( "Open Sans", 12, QFont.Bold ) )

    # Sends user info after start is selected
    def start( self, main_window ):
        # first see which page is selected
        # then grab the parameters the user selected and send to uC -> i2c obj
        # poll to read values of the transducers that the uC sends
        # loop until the user hits stop in the GUI
        page_number = main_window.stacked_widget.currentIndex()
        data = []

        if( page_number == Page.BJT_ON ):
            data.append( float( main_window.comboBox_vc_off.currentText() ) )
            data.append( float( main_window.comboBox_rc_off.currentText() ) )
            data.append( float( main_window.comboBox_vb_off.currentText() ) )
            date.append( float( main_window.comboBox_rb_off.currentText() ) )
        elif( page_number == Page.MOSFET_ON ):
            data.append( float( main_window.comboBox_vd_off.currentText() ) )
            data.append( float( main_window.comboBox_rd_off.currentText() ) )
            data.append( float( main_window.comboBox_vg_off.currentText() ) )
        elif( page_number == Page.DIODE_ON ):
            data.append( float( main_window.comboBox_vin_off.currentText() ) )
            data.append( float( main_window.comboBox_r_off.currentText() ) )

        # Create I2C instance

    def add_functionality( self, main_window ):
        self.init_labels( main_window )
        self.init_buttons( main_window )
        self.init_combos( main_window )
        self.init_easter_eggs( main_window )
        self.init_menubar( main_window )

########################################################################
###

    def init_menubar( self, main_window ):
        main_window.action_help.triggered.connect( HelpDialog ) 
        main_window.action_VNC.triggered.connect( VNCDialog ) 
        main_window.action_about.triggered.connect( AboutDialog )
        main_window.action_exit.triggered.connect( QuitDialog ) 
        main_window.action_info.triggered.connect( InfoDialog ) 

########################################################################
### Combo Button event init and handlers                             ###
########################################################################
    def init_combos( self, main_window ):
        # BJT
        main_window.comboBox_vc_off.addItems( [ x + " V" for x in BJT.Vcc ] )
        main_window.comboBox_rc_off.addItems( [ x + kOhm for x in BJT.Rc ] )
        main_window.comboBox_vb_off.addItems( [ x + " V" for x in BJT.Vbb ] )
        main_window.comboBox_rb_off.addItems( [ x + kOhm for x in BJT.Rb ] )
        
        main_window.comboBox_vc_on.setEnabled( False )
        main_window.comboBox_rc_on.setEnabled( False )
        main_window.comboBox_vb_on.setEnabled( False )
        main_window.comboBox_rb_on.setEnabled( False )
        main_window.comboBox_vbe_on.setEnabled( False ) 
        main_window.comboBox_vce_on.setEnabled( False )
        main_window.comboBox_vrb_on.setEnabled( False ) 
        main_window.comboBox_vrc_on.setEnabled( False )
        main_window.comboBox_ib_on.setEnabled( False ) 
        main_window.comboBox_ic_on.setEnabled( False )
        main_window.comboBox_ie_on.setEnabled( False )

        # MOSFET
        main_window.comboBox_vg_off.addItems( [ x + " V" for x in MOSFET.Vgg ] )
        main_window.comboBox_vd_off.addItems( [ x + " V" for x in MOSFET.Vdd ] )
        main_window.comboBox_rd_off.addItems( [ x + kOhm for x in MOSFET.Rd ] )

        main_window.comboBox_vg_on.setEnabled( False )
        main_window.comboBox_vd_on.setEnabled( False )
        main_window.comboBox_rd_on.setEnabled( False )      
        main_window.comboBox_vrd_on.setEnabled( False )      
        main_window.comboBox_vgs_on.setEnabled( False )
        main_window.comboBox_vds_on.setEnabled( False )
        main_window.comboBox_id_on.setEnabled( False )
 
        # Diode
        main_window.comboBox_r_off.addItems( [ x + kOhm for x in Diode.R ] )
        main_window.comboBox_vin_off.addItems( [ x + " V" for x in Diode.Vin ] )

        main_window.comboBox_r_on.setEnabled( False )
        main_window.comboBox_vin_on.setEnabled( False )
        main_window.comboBox_vr_on.setEnabled( False )
        main_window.comboBox_v_diode_on.setEnabled( False )
        main_window.comboBox_i_diode_on.setEnabled( False )
 
    def combo_start_bjt( self, main_window ):
        main_window.comboBox_vc_on.clear()
        main_window.comboBox_vc_on.addItems( [ main_window.comboBox_vc_off.currentText() ] )
        main_window.comboBox_rc_on.clear()
        main_window.comboBox_rc_on.addItems( [ main_window.comboBox_rc_off.currentText() ] )

        main_window.comboBox_vb_on.clear()
        main_window.comboBox_vb_on.addItems( [ main_window.comboBox_vb_off.currentText() ] )
        main_window.comboBox_rb_on.clear()
        main_window.comboBox_rb_on.addItems( [ main_window.comboBox_rb_off.currentText() ] )

        Vcc = float( main_window.comboBox_vc_off.currentText()[:-2] )
        Vbb = float( main_window.comboBox_vb_off.currentText()[:-2] )
        Rc = float( main_window.comboBox_rc_off.currentText()[:-3] )*1000
        Rb = float( main_window.comboBox_rb_off.currentText()[:-3] )*1000
        
        Vbe = 0.7;
        B = 100;
        Ib = ( Vbb - Vbe ) / Rb
        Ic = B * Ib
        Ie = Ib + Ic
        Vrb = Ib * Rb
        Vrc = Ic * Rc
        Vce = Vcc - Vrc
        if( Ib < 0 and Vce > 0 ):
            mode = "cutoff" 
            Ib = 0
            Ic = 0
            Ie = 0
            Vrb = 0
            Vrc = 0
            Vce = Vcc
        elif( Vce >= 0.5 ):
            mode = "forwardactive"
            Ib = Ib
        elif( Vce < 0.5 and Vce > 0 ):
            mode = "saturation"
            Ic = 10 * Ib
            Ie = Ib + Ic
            Vrb = Ib * Rb
            Vrc = Ic * Rc
            Vce = Vcc - Vrc

       
        # sensor values
        sensors = imp.bjt( Vbb, Vcc, Rb, Rc )

        _vce = abs( sensors[8] - sensors[9] )
        _vbe = abs( sensors[10] - sensors[9] ) 

        _vrc = abs( sensors[7] - sensors[8] )
        _vrb = abs( sensors[11] - sensors[10] )

        if mode == "cutoff":
            _vce = Vcc
            _vbe = Vbb
            _vrc = Vrc 
            _vrb = Vrb
        else:
            _vce = _vce - ( _vce - Vce ) / 2
            _vbe = _vbe - ( _vbe - Vbe ) / 2
            _vrc = _vrc - ( _vrc - Vrc ) / 2
            _vrb = _vrb - ( _vrb - Vrb ) / 2 
        #print(vce)
        #print(_vce)
        #print(vbe)
        #print(_vbe)
        #print(vrc)
        #print(_vrc)
        #print(vrb)
        #print(_vrb)
        '''
        if(_vrc == 0.0 and _vrb == 0.0):
            Vrb = _vrb
            Vrc = _vrc
            Ic = 0
            Ib = 0
            Ie = 0
        else:
            Vce = _vce - ( _vce - Vce ) / 2
            Vbe = _vbe - ( _vbe - Vbe ) / 2

            Vrb = _vrb - ( _vrb - Vrb ) / 2
            Vrc = _vrc - ( _vrc - Vrc ) / 2
        '''
        self.vce_swp = str( round( _vce, 2 ) ) + " V"
        self.vbe_swp = str( round( _vbe, 2 ) ) + " V"
        self.vrc_swp = str( round( _vrc, 2 ) ) + " V"
        self.vrb_swp = str( round( _vrb, 2 ) ) + " V"


        imp.SendCurrents( Ic, Ib )
        

        main_window.label_bjt_on_device.setPixmap( QtGui.QPixmap( Image.BJT[ mode ] ) )

        main_window.comboBox_vce_on.clear()
        main_window.comboBox_vce_on.addItems( [ str( round( Vce, 2 ) ) + " V"] )
        main_window.comboBox_vbe_on.clear()
        main_window.comboBox_vbe_on.addItems( [ str( round( Vbe, 2 ) ) + " V"] ) 
        main_window.comboBox_ib_on.clear()

        if Ib > 0.001:
            main_window.comboBox_ib_on.addItems( [ str( round( Ib, 3 ) ) + "mA"] ) 
        elif Ib > 1000:
            main_window.comboBox_ib_on.addItems( [ str( round( Ib / 1000, 3 ) ) + "A"] ) 
        else:
            main_window.comboBox_ib_on.addItems( [ str( round( Ib * 1000, 3 ) ) + "uA"] ) 

        main_window.comboBox_ic_on.clear()
 
        if Ic > 0.001:
            main_window.comboBox_ic_on.addItems( [ str( round( Ic, 3 ) ) + "mA"] ) 
        elif Ic > 1000:
            main_window.comboBox_ic_on.addItems( [ str( round( Ic / 1000, 3 ) ) + "A"] ) 
        else:
            main_window.comboBox_ic_on.addItems( [ str( round( Ic * 1000, 3 ) ) + "uA"] ) 
      
        main_window.comboBox_ie_on.clear()
  
        if Ie > 0.001:
            main_window.comboBox_ie_on.addItems( [ str( round( Ie, 3 ) ) + "mA"] ) 
        elif Ie > 1000:
            main_window.comboBox_ie_on.addItems( [ str( round( Ie / 1000, 3 ) ) + "A"] ) 
        else:
            main_window.comboBox_ie_on.addItems( [ str( round( Ie * 1000, 3 ) ) + "uA"] ) 
 
        main_window.comboBox_vrb_on.clear()
        main_window.comboBox_vrb_on.addItems( [ str( round( Vrb, 2 ) ) + " V"] ) 
        main_window.comboBox_vrc_on.clear()
        main_window.comboBox_vrc_on.addItems( [ str( round( Vrc, 2 ) ) + " V"] ) 
        

    def combo_stop_bjt( self, main_window ):
        imp.DiodeReturnToZero()
        pass

    def combo_start_mosfet( self, main_window ):
        main_window.comboBox_vd_on.clear()
        main_window.comboBox_vd_on.addItems( [ main_window.comboBox_vd_off.currentText() ] )
        main_window.comboBox_rd_on.clear()
        main_window.comboBox_rd_on.addItems( [ main_window.comboBox_rd_off.currentText() ] )
        main_window.comboBox_vg_on.clear()
        main_window.comboBox_vg_on.addItems( [ main_window.comboBox_vg_off.currentText() ] )

        vdd = float( main_window.comboBox_vd_off.currentText()[:-2] )
        vgg = float( main_window.comboBox_vg_off.currentText()[:-2] )
        rd = float( main_window.comboBox_rd_off.currentText()[:-3] )*1000
        print( "Vdd: " + str( vdd ) + "; Rd: " + str( rd ) )
        print( "Vgg: " + str( vgg ) )

        cox = 200 * 10 ** -9
        mu = 200
        w = 25 * 10 ** -6
        l = 10 ** -6
        vt = 0.7

        # Calculations for mode of op
        vgs = vgg
        ids = mu * cox * w / l * ( ( vgs - vt ) ** 2 ) / 2
        vrd = ids * rd
        vds = vdd - vrd

        if(vgs < vt):
            mode = "cutoff"
            ids = 0
        elif( vds < ( vgs - vt ) ):
            mode = "triode"
            ids = mu * cox * w/l * ( ( ( vgs - vt ) * vds ) - ( ( vds ** 2 ) / 2) )
        else:
            mode = "saturation"
            ids = mu * cox * w / l * ( ( vgs - vt ) ** 2 ) / 2
        vrd = ids * rd
        vds = vdd - vrd
 
        # sensor calculations             
        sensors = imp.mosfet1( vdd, vgg, rd )

        _vgs = abs( sensors[6] - sensors[3] )
        _vds = abs( sensors[4] - sensors[3] )
        _vrd = abs( sensors[5] - sensors[4] )
        print("vgs,vds,vrd")
        print(vgs)
        print(_vgs)
        print(vds)
        print(_vds)
        print(vrd)
        print(_vrd)
        if(vgg < vt):
            _vds = vdd
            _vgs = vgg
            _vrd = 0
        else:
            _vds = _vds - ( _vds - vds ) / 2
            _vgs = _vgs - ( _vgs - vgs ) / 2
            _vrd = _vrd - ( _vrd - vrd ) / 2
         
        self.vrd_swp = str( round( _vrd, 2 ) ) + " V"
        self.vds_swp = str( round( _vds, 2 ) ) + " V"
        self.vgs_swp = str( round( _vgs, 2 ) ) + " V"

        imp.SendCurrents( ids )

        main_window.label_mosfet_on_device.setPixmap( QtGui.QPixmap( Image.MOSFET[ mode ] ) )

        main_window.comboBox_vds_on.clear()
        main_window.comboBox_vds_on.addItems( [ str( round( vds, 2 ) ) + " V" ] )
        main_window.comboBox_vgs_on.clear()
        main_window.comboBox_vgs_on.addItems( [ str( round( vgs, 2 ) ) + " V" ] )
        main_window.comboBox_id_on.clear()

        if ids > 0.001:
            main_window.comboBox_id_on.addItems( [ str( round( ids, 3 ) ) + "mA" ] )
        elif ids > 1000:
            main_window.comboBox_id_on.addItems( [ str( round( ids / 1000, 3 ) ) + "A" ] )
        else:
            main_window.comboBox_id_on.addItems( [ str( round( ids * 1000, 3 ) ) + "uA" ] )


        main_window.comboBox_vrd_on.clear()
        main_window.comboBox_vrd_on.addItems( [ str( round( vrd, 2 ) ) + " V" ]  )

    def combo_stop_mosfet( self, main_window ):
        imp.DiodeReturnToZero()
        pass
 
    def combo_start_diode( self, main_window ):
        # Inputs
        main_window.comboBox_vin_on.clear()
        main_window.comboBox_vin_on.addItems( [ main_window.comboBox_vin_off.currentText() ] )
        main_window.comboBox_r_on.clear()
        main_window.comboBox_r_on.addItems( [ main_window.comboBox_r_off.currentText() ] )

        # Set function
        vin = float( main_window.comboBox_vin_off.currentText()[:-2] )
        rd = float( main_window.comboBox_r_off.currentText()[:-3] )
        print( "Vin: " + str( vin ) + "; Rd: " + str( rd ) )

        vt = 0.7

        vd = vt
        vr = vin - vd
        ir = vr / rd
         
        sensors = imp.diode( vin, rd )
        
        _vr = abs( sensors[2] - sensors[1] )
        _vd = abs( sensors[1] - sensors[0] )

        if( vin < vt ):
            print("cutoff")
            ir = 0.0
            vr = 0.0
            vd = vin
        else:
            if _vd < vt:
                _vd = _vd + ( vt - _vd ) / 2        
            elif vd > vt:
                _vd = _vd - ( _vd - vt ) / 2        
            if _vr < ( vin - vt ):
                _vr = _vr + ( ( vin - vt ) - _vr ) / 2
            elif _vr > ( vin - vt ):
                _vr = _vr - ( _vr - ( vin - vt ) ) / 2
            _vd = _vd - ( _vd - vd ) / 2
            _vr = _vr - ( _vr - vr ) / 2

        self.vd_swp = str( round( _vd, 2 ) ) + " V"
        self.vr_swp = str( round( _vr, 2 ) ) + " V"
        
        imp.SendCurrents( ir )
 
        mode = "on"
        if vin > vd:
            mode = "forwardbias"
        else:
            ## Ignore term. It's cutoff
            mode = "reversebias"

        main_window.label_diode_on_device.setPixmap( QtGui.QPixmap( Image.Diode[ mode ] ) )

        # Outputs
        main_window.comboBox_vr_on.clear()
        main_window.comboBox_vr_on.addItems( [ str( round( vr, 2 ) ) + " V" ] )
        main_window.comboBox_v_diode_on.clear()
        main_window.comboBox_v_diode_on.addItems( [ str( round( vd, 2 ) ) + " V" ] )
        main_window.comboBox_i_diode_on.clear()

        if ir < 0.001:
            main_window.comboBox_i_diode_on.addItems( [ str( round( ir, 3 ) ) + "mA"] )
        elif ir > 1000:
            main_window.comboBox_i_diode_on.addItems( [ str( round( ir / 1000, 3 ) ) + "A"] )
        else:
            main_window.comboBox_i_diode_on.addItems( [ str( round( ir * 1000, 3 ) ) + "mA"] )

    
    def combo_stop_diode( self, main_window ):
        imp.DiodeReturnToZero()
        pass

########################################################################
### Push Button event init and handlers                              ###
########################################################################
    def disable_start( self ):
        self.main_window.pushButton_start_bjt_off.setEnabled( False )
        self.main_window.pushButton_start_mosfet_off.setEnabled( False )
        self.main_window.pushButton_start_diode_off.setEnabled( False )

    def enable_start( self ):
        self.main_window.pushButton_start_bjt_off.setEnabled( True )
        self.main_window.pushButton_start_mosfet_off.setEnabled( True )
        self.main_window.pushButton_start_diode_off.setEnabled( True )

    def disable_stop( self ):
        self.main_window.pushButton_stop_bjt_on.setEnabled( False )
        self.main_window.pushButton_stop_mosfet_on.setEnabled( False ) 
        self.main_window.pushButton_stop_diode_on.setEnabled( False )

    def enable_stop( self ):
        self.main_window.pushButton_stop_bjt_on.setEnabled( True )
        self.main_window.pushButton_stop_mosfet_on.setEnabled( True )
        self.main_window.pushButton_stop_diode_on.setEnabled( True )
    
    def bjt_update_vc(self, main_window):
        print(float(main_window.comboBox_vc_off.currentText()[:-2]))
        try:
            if(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3):
                main_window.comboBox_vb_off.clear()
                main_window.comboBox_vb_off.addItems( [ x + " V" for x in ["0.8","1.6","12","0.6"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12):
                main_window.comboBox_vb_off.clear()
                main_window.comboBox_vb_off.addItems( [ x + " V" for x in ["0.8","1","2.2","0.6"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1):
                main_window.comboBox_vb_off.clear()
                main_window.comboBox_vb_off.addItems( [ x + " V" for x in ["0.8","1","0.6"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5):
                main_window.comboBox_vb_off.clear()
                main_window.comboBox_vb_off.addItems( [ x + " V" for x in ["0.8","1.4","3","0.6"]])
        except:
            print("setting up")

    def bjt_update_vb(self, main_window):
        try:
            if(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["10","20","41","55"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.6):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["82","86","92","100"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 12):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["46","57","67","87"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.0):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["11","27","50","100"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 2.2):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["40","51","78","90"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["6","16","42","61"]])    
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["42","50","54","67","91"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.0):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["91","92","93","94"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["11","13","41","72"]]) 
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.4):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["28","42","56","71"]])       
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 3.0):
                main_window.comboBox_rb_off.clear()
                main_window.comboBox_rb_off.addItems([x + kOhm for x in ["92","93","94","95"]]) 
        except:
            print("setting up")
    def bjt_update_rb(self, main_window):
        try:
            print("updatingRc HeHe")
            if(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 10):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 20):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 41):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 55):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","10","17"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.6 and float(main_window.comboBox_rb_off.currentText()[:-3]) ==82 ):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.6 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 86):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.6 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 92):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.6 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 100):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 12 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 46):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 12 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 57):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 12 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 67):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 3.3 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 12 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 87):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","3","5"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 11):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","3","4"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 27 ):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","5","10"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 50):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["3","15","19"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 100):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["10","25","39"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 6):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","3","5"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 16):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["6","11","19"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 42):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["15","30","50"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 61):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["10","53","73"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 2.2 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 40):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 2.2 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 51):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","3","4"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 2.2 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 78):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","3","6"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 12 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 2.2 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 90):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["2","4","7"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 42):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3","4"]]) 
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 50):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3","4"]])                               
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 54):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3","4","5"]])      
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 67):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3","4","5"]])     
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 91):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3","4","5"]])    
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 91):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])   
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 92):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 93):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])    
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 94):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])    
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 11):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["2","3","4","5"]])   
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 13):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["3","4","5","6"]])    
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 41):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["17","18","19","20"]])    
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 72):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["32","33","34","35"]])    
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.4 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 28):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2"]])   
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.4 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 42):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2","3"]])   
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.4 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 56):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["2","3","4"]])   
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 1.4 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 71):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["3","4","5"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 3.0 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 92):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 3.0 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 93):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2"]])    
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 3.0 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 94):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2"]])
            elif(float(main_window.comboBox_vc_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vb_off.currentText()[:-2]) == 3.0 and float(main_window.comboBox_rb_off.currentText()[:-3]) == 95):
                main_window.comboBox_rc_off.clear()
                main_window.comboBox_rc_off.addItems([x + kOhm for x in ["1","2"]]) 
        except:
            print("setting up")

    def mosfet_update(self, main_window):
        try:
            if(float(main_window.comboBox_vd_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 0.8):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd0_80_8])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.0):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd0_81_0])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.2):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd0_81_2])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 0.8 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.4):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd0_81_4])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 0.8):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd1_00_8])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.0):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd1_01_0])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.2):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd1_01_2])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 1.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.4):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd1_01_4])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 3.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 0.8):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd3_00_8])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 3.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.0):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd3_01_0])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 3.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.2):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd3_01_2])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 3.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.4):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd3_01_4])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 0.8):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd5_00_8])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.0):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd5_01_0])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.2):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd5_01_2])
            elif(float(main_window.comboBox_vd_off.currentText()[:-2]) == 5.0 and float(main_window.comboBox_vg_off.currentText()[:-2]) == 1.4):
                main_window.comboBox_rd_off.clear()
                main_window.comboBox_rd_off.addItems([x + kOhm for x in MOSFET.Rd5_01_4])
        except:
            print("setting up")
    def init_buttons( self, main_window ):
        main_window.comboBox_vc_off.currentIndexChanged.connect(lambda: self.bjt_update_vc(main_window))
        main_window.comboBox_vb_off.currentIndexChanged.connect(lambda: self.bjt_update_vb(main_window))
        main_window.comboBox_rb_off.currentIndexChanged.connect(lambda: self.bjt_update_rb(main_window))
        main_window.comboBox_vd_off.currentIndexChanged.connect(lambda: self.mosfet_update(main_window))
        main_window.comboBox_vg_off.currentIndexChanged.connect(lambda: self.mosfet_update(main_window))
        main_window.pushButton_back_bjt_off.clicked.connect( lambda: self.push_button_back( main_window ) )
        main_window.pushButton_back_mosfet_off.clicked.connect( lambda: self.push_button_back( main_window ) )
        main_window.pushButton_back_diode_off.clicked.connect( lambda: self.push_button_back( main_window ) )

        main_window.pushButton_start_bjt_off.clicked.connect( lambda: self.push_button_bjt_start( main_window ) )
        main_window.pushButton_stop_bjt_on.clicked.connect( lambda: self.push_button_bjt_stop( main_window ) )
        main_window.pushButton_start_mosfet_off.clicked.connect( lambda: self.push_button_mosfet_start( main_window ) )
        main_window.pushButton_stop_mosfet_on.clicked.connect( lambda: self.push_button_mosfet_stop( main_window ) )
        main_window.pushButton_start_diode_off.clicked.connect( lambda: self.push_button_diode_start( main_window ) )
        main_window.pushButton_stop_diode_on.clicked.connect( lambda: self.push_button_diode_stop( main_window ) )

        main_window.pushButton_bjt_switch.clicked.connect( lambda: self.switch_bjt( main_window ) )
        main_window.pushButton_mosfet_switch.clicked.connect( lambda: self.switch_mosfet( main_window ) )
        main_window.pushButton_diode_switch.clicked.connect( lambda: self.switch_diode( main_window ) )
        main_window.pushButton_bjt_switch.setText( "Theoretical\nVoltages" )
        main_window.pushButton_mosfet_switch.setText( "Theoretical\nVoltages" )
        main_window.pushButton_diode_switch.setText( "Theoretical\nVoltages" )

    def switch_bjt( self, main_window ):
        temp = self.vrc_swp
        self.vrc_swp = main_window.comboBox_vrc_on.currentText()
        main_window.comboBox_vrc_on.clear()
        main_window.comboBox_vrc_on.addItems( [ temp ] )

        temp = self.vrb_swp
        self.vrb_swp = main_window.comboBox_vrb_on.currentText()
        main_window.comboBox_vrb_on.clear()
        main_window.comboBox_vrb_on.addItems( [ temp ] )

        temp = self.vce_swp
        self.vce_swp = main_window.comboBox_vce_on.currentText()
        main_window.comboBox_vce_on.clear()
        main_window.comboBox_vce_on.addItems( [ temp ] )

        temp = self.vbe_swp
        self.vbe_swp = main_window.comboBox_vbe_on.currentText()
        main_window.comboBox_vbe_on.clear()
        main_window.comboBox_vbe_on.addItems( [ temp ] )
       
        self.switch_text( main_window.pushButton_bjt_switch )    

 
    def switch_mosfet( self, main_window ):
        temp = self.vrd_swp
        self.vrd_swp = main_window.comboBox_vrd_on.currentText()
        main_window.comboBox_vrd_on.clear()
        main_window.comboBox_vrd_on.addItems( [ temp ] )

        temp = self.vds_swp
        self.vds_swp = main_window.comboBox_vds_on.currentText()
        main_window.comboBox_vds_on.clear()
        main_window.comboBox_vds_on.addItems( [ temp ] )

        temp = self.vgs_swp
        self.vgs_swp = main_window.comboBox_vgs_on.currentText()
        main_window.comboBox_vgs_on.clear()
        main_window.comboBox_vgs_on.addItems( [ temp ] )
       
        self.switch_text( main_window.pushButton_mosfet_switch )    


    def switch_diode( self, main_window ):
        temp = self.vd_swp
        self.vd_swp = main_window.comboBox_v_diode_on.currentText()
        main_window.comboBox_v_diode_on.clear()
        main_window.comboBox_v_diode_on.addItems( [ temp ] )

        temp = self.vr_swp
        self.vr_swp = main_window.comboBox_vr_on.currentText()
        main_window.comboBox_vr_on.clear()
        main_window.comboBox_vr_on.addItems( [ temp ] )

        self.switch_text( main_window.pushButton_diode_switch )    

    def switch_text( self, obj ):
        if obj.text() == "System\nVoltages":
            obj.setText( "Theoretical\nVoltages" )
        else:
            obj.setText( "System\nVoltages" )

    def push_button_back( self, main_window ):
        main_window.stacked_widget.setCurrentIndex( Page.MAIN.value )
        self.banner( "A.C.L.A." )

    def push_button_bjt_start( self, main_window ):
        main_window.stacked_widget.setCurrentIndex( Page.BJT_ON.value )
        self.disable_stop()
        self.enable_stop_timer()
        self.combo_start_bjt( main_window )

    def push_button_bjt_stop( self, main_window ):
        main_window.stacked_widget.setCurrentIndex( Page.BJT_OFF.value )
        self.disable_start()
        self.enable_start_timer()
        self.combo_stop_bjt( main_window )

    def push_button_mosfet_start( self, main_window ):
        main_window.stacked_widget.setCurrentIndex( Page.MOSFET_ON.value )
        self.disable_stop()
        self.enable_stop_timer()
        self.combo_start_mosfet( main_window )

    def push_button_mosfet_stop( self, main_window ):
        main_window.stacked_widget.setCurrentIndex( Page.MOSFET_OFF.value )
        self.disable_start()
        self.enable_start_timer()
        self.combo_stop_mosfet( main_window )

    def push_button_diode_start( self, main_window ):
        main_window.stacked_widget.setCurrentIndex( Page.DIODE_ON.value )
        self.disable_stop()
        self.enable_stop_timer()
        self.combo_start_diode( main_window )

    def push_button_diode_stop( self, main_window ):
        main_window.stacked_widget.setCurrentIndex( Page.DIODE_OFF.value )
        self.disable_start()
        self.enable_start_timer()
        self.combo_stop_diode( main_window )

########################################################################
### Timer
########################################################################
    def enable_start_timer( self, time=4000 ):
        self.start_timer = QtCore.QTimer( self )
        self.start_timer.timeout.connect( self.enable_start )
        self.start_timer.setSingleShot( True )
        self.start_timer.setInterval( 100 )
        self.start_timer.start( time )

    def enable_stop_timer( self, time=8000 ):
        self.stop_timer = QtCore.QTimer( self )
        self.stop_timer.timeout.connect( self.enable_stop )
        self.stop_timer.setSingleShot( True )
        self.stop_timer.setInterval( 100 )
        self.stop_timer.start( time )

########################################################################
### Label event init and handlers                                    ###
########################################################################
    def init_labels( self, main_window ):
        clickable( main_window.label_main_bjt ).connect( self.label_bjt ) 
        clickable( main_window.label_main_mosfet ).connect( self.label_mosfet ) 
        clickable( main_window.label_main_diode ).connect( self.label_diode ) 

    def label_bjt( self ):
        self.main_window.stacked_widget.setCurrentIndex( Page.BJT_OFF.value )
        self.banner( "BJT" )

    def label_mosfet( self ):
        self.main_window.stacked_widget.setCurrentIndex( Page.MOSFET_OFF.value )
        self.banner( "MOSFET" )

    def label_diode( self ):
        self.main_window.stacked_widget.setCurrentIndex( Page.DIODE_OFF.value )
        self.banner( "Diode" )
        
########################################################################
### Easter Egg  event init and handlers                              ###
########################################################################

    def init_easter_eggs( self, main_window ):
        main_window.label_easter_egg_0.setText( "" ) 
        main_window.label_easter_egg_1.setText( "" ) 
        main_window.label_easter_egg_2.setText( "" ) 
        main_window.label_easter_egg_3.setText( "" ) 

        clickable( main_window.label_easter_egg_0 ).connect( self.easter_egg_0 ) 
        clickable( main_window.label_easter_egg_1 ).connect( self.easter_egg_1 ) 
        clickable( main_window.label_easter_egg_2 ).connect( self.easter_egg_2 ) 
        clickable( main_window.label_easter_egg_3 ).connect( self.easter_egg_3 ) 
        clickable( main_window.label_diode_off_device ).connect( self.easter_egg_4 ) 

    def easter_egg_0( self ):
        easter_egg_code.append( 0 )
    def easter_egg_1( self ):
        easter_egg_code.append( 1 )    
    def easter_egg_2( self ):
        easter_egg_code.append( 2 )
    def easter_egg_3( self ):
        easter_egg_code.append( 3 )
    def easter_egg_4( self ):
        if easter_egg_code == [ 0, 1, 2, 3, 2, 1, 0 ]:
            self.easter_egg()
        easter_egg_code.clear()
    def easter_egg( self ):
        print( "Easter Egg unlocked!" )
    
########################################################################
### terminate
###     Safely exits the application
########################################################################
def terminate( self ):
    # Resets stepper motor
    page_number = window.main_window.stacked_widget.currentIndex()

    if page_number == Page.BJT_ON:
        window.main_window.combo_stop_bjt( window.main_window )
    elif page_number == Page.MOSFET_ON:
        window.main_windowcombo_stop_mosfet( window.main_window )
    elif page_number == Page.DIODE_ON:
        window.main_window.combo_stop_diode( window.main_window )    

    if not debug:
        from subprocess import call
        call( "shutdown now", shell=True )
    else:
        for obj in open_dialog:
            obj.close()
        sys.exit()        

########################################################################
### clickable                                                        ###
###     Installs an event handler into any QObject (QLabels for out  ###
###     purposes most liekly) so that they all can function as       ###
###     push buttons...                                              ###
########################################################################
def clickable( widget ):
    class Filter( QtCore.QObject ):
        clicked = QtCore.pyqtSignal()

        def eventFilter( self, obj, event ):
            if obj == widget and event.type() == QtCore.QEvent.MouseButtonRelease and obj.rect().contains( event.pos() ):
                self.clicked.emit()
                return True
            return False
    filter = Filter( widget )
    widget.installEventFilter( filter )
    return filter.clicked

########################################################################
### Easier access to specific menu pages                             ###
########################################################################
class Page( Enum ):
    MAIN = 0
    BJT_OFF = 1
    BJT_ON = 2
    MOSFET_OFF = 3
    MOSFET_ON = 4
    DIODE_OFF = 5
    DIODE_ON = 6

########################################################################
###
########################################################################
open_dialog = []

class Dialog( QDialog ):
    def __init__( self, dialog_name ):
        super().__init__()
       
        for obj in open_dialog:
            obj.close()

        self.filename = uiDir + dialog_name + ".ui"

        self.dialog = uic.loadUi( self.filename, self )
        self.dialog.setFixedSize( self.width, self.height )

        self.dialog.show()     

        open_dialog.append( self.dialog )

        self.showFullScreen()

class AboutDialog( Dialog ):
    def __init__( self ):        
        self.width = 800 
        self.height = 480
        
        super().__init__( "about" )

        self.dialog.button_box_close.buttons()[0].clicked.connect( self.close )


class VNCDialog( Dialog ):
    def __init__( self ):        
        self.width = 600
        self.height = 360

        super().__init__( "vnc" )

        self.dialog.button_box_close.buttons()[0].clicked.connect( self.close )

class InfoDialog( Dialog ):
    def __init__( self ):
        self.width = 800
        self.height = 480
        self.page_num = 0

        super().__init__( "info" )


        self.stacked_widget.setCurrentIndex( self.page_num )
        self.pushButton_back.clicked.connect( self.close )
        self.pushButton_next.clicked.connect( self.next ) 
        self.pushButton_previous.clicked.connect( self.previous )

    def next( self ):
        self.page_num = ( self.page_num + 1 ) % self.stacked_widget.count()
        self.stacked_widget.setCurrentIndex( self.page_num )

    def previous( self ):
        self.page_num = ( self.page_num - 1 ) % self.stacked_widget.count()
        self.stacked_widget.setCurrentIndex( self.page_num )


directory = "img/tutorial/"
image = [ directory ] * 11 
for i in range( len( image ) ):
    image[i] = image[i] + "info_" + str( i ) + ".png"
class HelpDialog( Dialog ):
    def __init__( self ):        
        self.width = 800
        self.height = 480
        self.page_num = 0

        super().__init__( "help" )

        self.stacked_widget.setCurrentIndex( self.page_num )

        self.pushButton_back.clicked.connect( self.close )
        self.pushButton_next.clicked.connect( self.next ) 
        self.pushButton_previous.clicked.connect( self.previous )

        self.label_0.setPixmap( QtGui.QPixmap( image[0] ) )
        self.label_1.setPixmap( QtGui.QPixmap( image[1] ) )
        self.label_2.setPixmap( QtGui.QPixmap( image[2] ) )
        self.label_3.setPixmap( QtGui.QPixmap( image[3] ) )
        self.label_4.setPixmap( QtGui.QPixmap( image[4] ) )
        self.label_5.setPixmap( QtGui.QPixmap( image[5] ) )
        self.label_6.setPixmap( QtGui.QPixmap( image[6] ) )
        self.label_7.setPixmap( QtGui.QPixmap( image[7] ) )
        self.label_8.setPixmap( QtGui.QPixmap( image[8] ) )
        self.label_9.setPixmap( QtGui.QPixmap( image[9] ) )
        self.label_10.setPixmap( QtGui.QPixmap( image[10] ) )

    def next( self ):
        self.page_num = ( self.page_num + 1 ) % self.stacked_widget.count()
        self.stacked_widget.setCurrentIndex( self.page_num )

    def previous( self ):
        self.page_num = ( self.page_num - 1 ) % self.stacked_widget.count()
        self.stacked_widget.setCurrentIndex( self.page_num )

class QuitDialog( Dialog ):
    def __init__( self ):
        self.width = 388
        self.height = 161

        super().__init__( "quit" )

        self.dialog.button_box_terminate.buttons()[0].clicked.connect( terminate )
        self.dialog.button_box_terminate.buttons()[1].clicked.connect( self.close )


########################################################################
### main                                                             ###
###     Launches the GUI, initializing everything orderly            ###
###     							     ###
###     flags:							     ###
###		-d	:	debug mode			     ###
########################################################################
def main():
    global window
    global debug
    debug = False

    if "-d" in sys.argv:
        debug = True 
   
    app = QApplication( sys.argv )
    window = MainWindow()
    sys.exit( app.exec() )

if __name__ == "__main__":
    main() 
